@@ -1,4 +1,4 @@
-# $Id: SQL.pm 494 2008-06-10 23:13:56Z bchoate $
+# $Id: SQL.pm 564 2009-02-05 00:27:19Z athomason $
 
 package Data::ObjectDriver::SQL;
 use strict;
@@ -71,23 +71,27 @@ sub as_sql {
     $sql .= 'FROM ';
 
     ## Add any explicit JOIN statements before the non-joined tables.
+    my %joined;
+    my @from = @{ $stmt->from || [] };
     if ($stmt->joins && @{ $stmt->joins }) {
         my $initial_table_written = 0;
         for my $j (@{ $stmt->joins }) {
             my($table, $joins) = map { $j->{$_} } qw( table joins );
             $table = $stmt->_add_index_hint($table); ## index hint handling
             $sql .= $table unless $initial_table_written++;
+            $joined{$table}++;
             for my $join (@{ $j->{joins} }) {
                 $sql .= ' ' .
                         uc($join->{type}) . ' JOIN ' . $join->{table} . ' ON ' .
                         $join->{condition};
             }
         }
-        $sql .= ', ' if @{ $stmt->from };
+        @from = grep { ! $joined{ $_ } } @from;
+        $sql .= ', ' if @from;
     }
 
-    if ($stmt->from && @{ $stmt->from }) {
-        $sql .= join ', ', map { $stmt->_add_index_hint($_) } @{ $stmt->from };
+    if (@from) {
+        $sql .= join ', ', map { $stmt->_add_index_hint($_) } @from;
     }
 
     $sql .= "\n";
@@ -267,7 +271,7 @@ sub _add_index_hint {
     my $hint = $stmt->index_hint->{$tbl_name};
     return $tbl_name unless $hint && ref($hint) eq 'HASH';
     if ($hint->{list} && @{ $hint->{list} }) {
-        return $tbl_name . ' ' . uc($hint->{type} || 'USE') . ' INDEX (' . 
+        return $tbl_name . ' ' . uc($hint->{type} || 'USE') . ' INDEX (' .
                 join (',', @{ $hint->{list} }) .
                 ')';
     }
@@ -295,7 +299,7 @@ Data::ObjectDriver::SQL - an SQL statement
     my $sth = $dbh->prepare($sql->as_sql);
     $sth->execute(@{ $sql->{bind} });
     my @values = $sth->selectrow_array();
-    
+
     my $obj = SomeObject->new();
     $obj->set_columns(...);
 
@@ -573,7 +577,7 @@ be any of the structures described above for the C<add_where()> method.
 
 =head2 C<$sql-E<gt>add_index_hint($table, \@hints)>
 
-Addes the index hint into a C<SELECT> query. The structure for the set of 
+Addes the index hint into a C<SELECT> query. The structure for the set of
 C<\@hints> are arrayref of hashrefs containing these members:
 
 =over 4
