@@ -1,4 +1,4 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -259,6 +259,11 @@ sub init_core_registry {
                 label =>
                     'This module is required in mt-search.cgi if you are running Movable Type on Perl older than Perl 5.8.',
             },
+            'XML::Parser' => {
+                link => 'http://search.cpan.org/dist/Text-Balanced',
+                label =>
+                    'This module required for action streams.',
+            },
         },
         required_packages => {
             'Image::Size' => {
