@@ -1,4 +1,4 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -348,7 +348,8 @@ sub as_html {
     my $text = sprintf '<a href="%s">%s</a>',
         MT::Util::encode_html($asset->url),
         MT::Util::encode_html($fname);
-    return $asset->enclose($text);
+    my $app = MT->instance;
+    return $app->param('edit_field') =~ /^customfield/ ? $asset->enclose($text) : $text;
 }
 
 sub enclose {
