@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Blog.pm 3803 2009-06-08 03:20:26Z fumiakiy $
+# $Id: Blog.pm 4145 2009-08-17 00:38:39Z dphillips $
 
 package MT::Blog;
 
@@ -286,6 +286,7 @@ sub create_default_templates {
                 $map->template_id($tmpl->id);
                 $map->file_template($m->{file_template}) if $m->{file_template};
                 $map->blog_id($tmpl->blog_id);
+                $map->build_type( $m->{build_type} ) if defined $m->{build_type};
                 $map->save;
             }
         }
@@ -499,7 +500,18 @@ sub has_archive_type {
     my $blog = shift;
     my ($type) = @_;
     my %at = map { lc $_ => 1 } split(/,/, $blog->archive_type);
-    return exists $at{lc $type} ? 1 : 0;
+    return 0 unless exists $at{lc $type};
+
+    my $result = 0;
+    require MT::TemplateMap;
+    my @maps = MT::TemplateMap->load({blog_id => $blog->id,
+                                      archive_type => $type});
+    return 0 unless @maps;
+    require MT::PublishOption;
+    foreach my $map (@maps) {  
+        $result++ if $map->build_type != MT::PublishOption::DISABLED();
+    }
+    return $result;
 }
 
 sub accepts_registered_comments {
@@ -579,6 +591,8 @@ sub clone_with_children {
     $new_blog->name($blog_name ? $blog_name : MT->translate("Clone of [_1]", $blog->name));
     delete $new_blog->{column_values}->{id};
     delete $new_blog->{changed_cols}->{id};
+    $new_blog->modified_on(undef);
+    $new_blog->created_on(undef);
     $new_blog->save or die $new_blog->errstr;
     $new_blog_id = $new_blog->id;
     $callback->(MT->translate("Cloned blog... new id is [_1].",
@@ -591,14 +605,16 @@ sub clone_with_children {
         $callback->($state, "perms");
         require MT::Permission;
         $iter = MT::Permission->load_iter({blog_id => $old_blog_id});
+
         while (my $perm = $iter->()) {
             $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'perms')
                 if $counter && ($counter % 100 == 0);
             $counter++;
-            delete $perm->{column_values}->{id};
-            delete $perm->{changed_cols}->{id};
-            $perm->blog_id($new_blog_id);
-            $perm->save or die $perm->errstr;
+            my $new_perm = $perm->clone();
+            delete $new_perm->{column_values}->{id};
+            delete $new_perm->{changed_cols}->{id};
+            $new_perm->blog_id($new_blog_id);
+            $new_perm->save or die $new_perm->errstr;
         }
         $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'perms');
     }
@@ -614,10 +630,11 @@ sub clone_with_children {
             $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'assoc')
                 if $counter && ($counter % 100 == 0);
             $counter++;
-            delete $assoc->{column_values}->{id};
-            delete $assoc->{changed_cols}->{id};
-            $assoc->blog_id($new_blog_id);
-            $assoc->save or die $assoc->errstr;
+            my $new_assoc = $assoc->clone();
+            delete $new_assoc->{column_values}->{id};
+            delete $new_assoc->{changed_cols}->{id};
+            $new_assoc->blog_id($new_blog_id);
+            $new_assoc->save or die $new_assoc->errstr;
         }
         $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'assoc');
     }
@@ -644,11 +661,12 @@ sub clone_with_children {
                 if $counter && ($counter % 100 == 0);
             $counter++;
             my $entry_id = $entry->id;
-            delete $entry->{column_values}->{id};
-            delete $entry->{changed_cols}->{id};
-            $entry->blog_id($new_blog_id);
-            $entry->save or die $entry->errstr;
-            $entry_map{$entry_id} = $entry->id;
+            my $new_entry = $entry->clone();
+            delete $new_entry->{column_values}->{id};
+            delete $new_entry->{changed_cols}->{id};
+            $new_entry->blog_id($new_blog_id);
+            $new_entry->save or die $new_entry->errstr;
+            $entry_map{$entry_id} = $new_entry->id;
         }
         $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'entries');
 
@@ -666,16 +684,17 @@ sub clone_with_children {
                 $counter++;
                 my $cat_id = $cat->id;
                 my $old_parent = $cat->parent;
-                delete $cat->{column_values}->{id};
-                delete $cat->{changed_cols}->{id};
-                $cat->blog_id($new_blog_id);
+                my $new_cat = $cat->clone();
+                delete $new_cat->{column_values}->{id};
+                delete $new_cat->{changed_cols}->{id};
+                $new_cat->blog_id($new_blog_id);
                 # temporarily wipe the parent association
                 # to avoid constraint issues.
-                $cat->parent(0);
-                $cat->save or die $cat->errstr;
-                $cat_map{$cat_id} = $cat->id;
+                $new_cat->parent(0);
+                $new_cat->save or die $new_cat->errstr;
+                $cat_map{$cat_id} = $new_cat->id;
                 if ($old_parent) {
-                    $cat_parents{$cat->id} = $old_parent;
+                    $cat_parents{$new_cat->id} = $old_parent;
                 }
             }
             # reassign the new category parents
@@ -699,12 +718,13 @@ sub clone_with_children {
                 $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'places')
                     if $counter && ($counter % 100 == 0);
                 $counter++;
-                delete $place->{column_values}->{id};
-                delete $place->{changed_cols}->{id};
-                $place->blog_id($new_blog_id);
-                $place->category_id($cat_map{$place->category_id});
-                $place->entry_id($entry_map{$place->entry_id});
-                $place->save or die $place->errstr;
+                my $new_place = $place->clone();
+                delete $new_place->{column_values}->{id};
+                delete $new_place->{changed_cols}->{id};
+                $new_place->blog_id($new_blog_id);
+                $new_place->category_id($cat_map{$place->category_id});
+                $new_place->entry_id($entry_map{$place->entry_id});
+                $new_place->save or die $new_place->errstr;
             }
             $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'places');
         }
@@ -720,15 +740,17 @@ sub clone_with_children {
                 $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'comments')
                     if $counter && ($counter % 100 == 0);
                 $counter++;
-                delete $comment->{column_values}->{id};
-                delete $comment->{changed_cols}->{id};
-                $comment->entry_id($entry_map{$comment->entry_id});
-                $comment->blog_id($new_blog_id);
-                $comment->save or die $comment->errstr;
+                
+                my $new_comment = $comment->clone();
+                delete $new_comment->{column_values}->{id};
+                delete $new_comment->{changed_cols}->{id};
+                $new_comment->entry_id($entry_map{$comment->entry_id});
+                $new_comment->blog_id($new_blog_id);
+                $new_comment->save or die $new_comment->errstr;
             }
             $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'comments');
         }
-
+        
         if ((!exists $classes->{'MT::ObjectTag'}) || $classes->{'MT::ObjectTag'}) {
             # conditionally do MT::ObjectTag since it is only
             # available with MT 3.3.
@@ -742,15 +764,53 @@ sub clone_with_children {
                     $callback->($state . " " . MT->translate("[_1] records processed...", $counter), "tags")
                         if $counter && ($counter % 100 == 0);
                     $counter++;
-                    delete $entry_tag->{column_values}->{id};
-                    delete $entry_tag->{changed_cols}->{id};
-                    $entry_tag->blog_id($new_blog_id);
-                    $entry_tag->object_id($entry_map{$entry_tag->object_id});
-                    $entry_tag->save or die $entry_tag->errstr;
+                    my $new_entry_tag = $entry_tag->clone();
+                    delete $new_entry_tag->{column_values}->{id};
+                    delete $new_entry_tag->{changed_cols}->{id};
+                    $new_entry_tag->blog_id($new_blog_id);
+                    $new_entry_tag->object_id($entry_map{$entry_tag->object_id});
+                    $new_entry_tag->save or die $new_entry_tag->errstr;
                 }
                 $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'tags');
             }
         }
+    } 
+    elsif ((!exists $classes->{'MT::Category'}) || $classes->{'MT::Category'}) {
+        # Cloning CATEGORY records
+        my $state = MT->translate("Cloning categories for blog...");
+        $callback->($state, "cats");
+        $counter = 0;
+        require MT::Category;
+        $iter = MT::Category->load_iter({ blog_id => $old_blog_id, class => '*' });
+        my %cat_parents;
+        while (my $cat = $iter->()) {
+            $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'cats')
+                if $counter && ($counter % 100 == 0);
+            $counter++;
+            my $cat_id = $cat->id;
+            my $old_parent = $cat->parent;
+            my $new_cat = $cat->clone();
+            delete $new_cat->{column_values}->{id};
+            delete $new_cat->{changed_cols}->{id};
+            $new_cat->blog_id($new_blog_id);
+            # temporarily wipe the parent association
+            # to avoid constraint issues.
+            $new_cat->parent(0);
+            $new_cat->save or die $new_cat->errstr;
+            $cat_map{$cat_id} = $new_cat->id;
+            if ($old_parent) {
+                $cat_parents{$new_cat->id} = $old_parent;
+            }
+        }
+        # reassign the new category parents
+        foreach (keys %cat_parents) {
+            my $cat = MT::Category->load($_);
+            if ($cat) {
+                $cat->parent($cat_map{$cat_parents{$cat->id}});
+                $cat->save or die $cat->errstr;
+            }
+        }
+        $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'cats');
     }
 
     if ((!exists $classes->{'MT::Trackback'}) || $classes->{'MT::Trackback'}) {
@@ -767,8 +827,9 @@ sub clone_with_children {
                 if $counter && ($counter % 100 == 0);
             $counter++;
             my $tb_id = $tb->id;
-            delete $tb->{column_values}->{id};
-            delete $tb->{changed_cols}->{id};
+            my $new_tb = $tb->clone();
+            delete $new_tb->{column_values}->{id};
+            delete $new_tb->{changed_cols}->{id};
 
             if ($tb->category_id) {
                 if (my $cid = $cat_map{$tb->category_id}) {
@@ -814,13 +875,13 @@ sub clone_with_children {
             # A trackback wasn't created when saving the entry/category,
             # (perhaps trackbacks are now disabled for the entry/category?)
             # so create one now
-            $tb->entry_id($entry_map{$tb->entry_id})
+            $new_tb->entry_id($entry_map{$tb->entry_id})
                 if $tb->entry_id && $entry_map{$tb->entry_id};
-            $tb->category_id($cat_map{$tb->category_id})
+            $new_tb->category_id($cat_map{$tb->category_id})
                 if $tb->category_id && $cat_map{$tb->category_id};
-            $tb->blog_id($new_blog_id);
-            $tb->save or die $tb->errstr;
-            $tb_map{$tb_id} = $tb->id;
+            $new_tb->blog_id($new_blog_id);
+            $new_tb->save or die $new_tb->errstr;
+            $tb_map{$tb_id} = $new_tb->id;
         }
         $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'tbs');
 
@@ -835,11 +896,12 @@ sub clone_with_children {
                 $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'pings')
                     if $counter && ($counter % 100 == 0);
                 $counter++;
-                delete $ping->{column_values}->{id};
-                delete $ping->{changed_cols}->{id};
-                $ping->tb_id($tb_map{$ping->tb_id});
-                $ping->blog_id($new_blog_id);
-                $ping->save or die $ping->errstr;
+                my $new_ping = $ping->clone();
+                delete $new_ping->{column_values}->{id};
+                delete $new_ping->{changed_cols}->{id};
+                $new_ping->tb_id($tb_map{$ping->tb_id});
+                $new_ping->blog_id($new_blog_id);
+                $new_ping->save or die $new_ping->errstr;
             }
             $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'pings');
         }
@@ -853,32 +915,34 @@ sub clone_with_children {
             { blog_id => $old_blog_id, type => { not => 'widgetset' } }
         );
         my $tmpl_processor = sub {
-            my ( $new_blog_id, $counter, $tmpl, $tmpl_map ) = @_;
+            my ( $new_blog_id, $counter, $tmpl, $new_tmpl, $tmpl_map ) = @_;
             $callback->($state . " " . MT->translate("[_1] records processed...", $$counter), 'tmpls')
                 if $counter && ($$counter % 100 == 0);
             my $tmpl_id = $tmpl->id;
             $$counter++;
-            delete $tmpl->{column_values}->{id};
-            delete $tmpl->{changed_cols}->{id};
+            delete $new_tmpl->{column_values}->{id};
+            delete $new_tmpl->{changed_cols}->{id};
             # linked_file won't be cloned for now because
             # new blog does not have site_path - breaks relative path
-            delete $tmpl->{column_values}->{linked_file};
-            delete $tmpl->{column_values}->{linked_file_mtime};
-            delete $tmpl->{column_values}->{linked_file_size};
-            $tmpl->blog_id($new_blog_id);
-            $tmpl->save or die $tmpl->errstr;
-            $tmpl_map->{$tmpl_id} = $tmpl->id;
+            delete $new_tmpl->{column_values}->{linked_file};
+            delete $new_tmpl->{column_values}->{linked_file_mtime};
+            delete $new_tmpl->{column_values}->{linked_file_size};
+            $new_tmpl->blog_id($new_blog_id);
+            $new_tmpl->save or die $new_tmpl->errstr;
+            $tmpl_map->{$tmpl_id} = $new_tmpl->id;
         };
         $counter = 0;
         while (my $tmpl = $iter->()) {
-            $tmpl_processor->($new_blog_id, \$counter, $tmpl, \%tmpl_map);
+            my $new_tmpl = $tmpl->clone();
+            $tmpl_processor->($new_blog_id, \$counter, $tmpl, $new_tmpl, \%tmpl_map);
         }
         $iter = MT::Template->load_iter(
             { blog_id => $old_blog_id, type => 'widgetset' }
         );
         while (my $tmpl = $iter->()) {
             my @old_widgets = split /,/, $tmpl->modulesets;
-            $tmpl_processor->($new_blog_id, \$counter, $tmpl, \%tmpl_map);
+            my $new_tmpl = $tmpl->clone();
+            $tmpl_processor->($new_blog_id, \$counter, $tmpl, $new_tmpl, \%tmpl_map);
             my @new_widgets;
             foreach ( @old_widgets ) {
                 if ( exists $tmpl_map{$_} ) {
@@ -892,8 +956,8 @@ sub clone_with_children {
                       && $global_widget->type eq 'widget'
                 }
             }
-            $tmpl->modulesets( join(',', @new_widgets) );
-            $tmpl->save;
+            $new_tmpl->modulesets( join(',', @new_widgets) );
+            $new_tmpl->save;
         }
         $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'tmpls');
 
@@ -906,11 +970,12 @@ sub clone_with_children {
             $callback->($state . " " . MT->translate("[_1] records processed...", $counter), 'tmplmaps')
                 if $counter && ($counter % 100 == 0);
             $counter++;
-            delete $map->{column_values}->{id};
-            delete $map->{changed_cols}->{id};
-            $map->template_id($tmpl_map{$map->template_id});
-            $map->blog_id($new_blog_id);
-            $map->save or die $map->errstr;
+            my $new_map = $map->clone();
+            delete $new_map->{column_values}->{id};
+            delete $new_map->{changed_cols}->{id};
+            $new_map->template_id($tmpl_map{$map->template_id});
+            $new_map->blog_id($new_blog_id);
+            $new_map->save or die $new_map->errstr;
         }
         $callback->($state . " " . MT->translate("[_1] records processed.", $counter), 'tmplmaps');
     }
