@@ -174,7 +174,49 @@ sub edit {
         $param->{authored_on_time} = $q->param('authored_on_time')
           || POSIX::strftime( "%H:%M:%S", @now );
     }
-
+    
+    ## show the necessary associated assets
+    if ($type eq 'entry' || $type eq 'page') {
+       require MT::Asset;
+       require MT::ObjectAsset;
+       my $assets =();
+       if ($q->param('reedit') && $q->param('include_asset_ids')) {
+           my $include_asset_ids = $app->param('include_asset_ids');
+           my @asset_ids = split(',', $include_asset_ids);
+           foreach my $asset_id (@asset_ids) {
+               my $asset = MT::Asset->load($asset_id);
+               if ($asset) {
+                   my $asset_1;
+                   if ($asset->class eq 'image') {
+                       $asset_1 = {asset_id => $asset->id, asset_name => $asset->file_name, asset_thumb => $asset->thumbnail_url(Width=>100)};
+                   } else {
+                       $asset_1 = {asset_id => $asset->id, asset_name => $asset->file_name};
+                   }
+                   push @{$assets}, $asset_1;
+               }
+           }
+       }
+       elsif ($q->param('asset_id') && !$id) {
+           my $asset = MT::Asset->load($q->param('asset_id'));
+           my $asset_1 = {asset_id => $asset->id, asset_name => $asset->file_name};
+           push @{$assets}, $asset_1;
+       }
+       elsif ($id) {
+           my @assets = MT::Asset->load({ class => '*' },
+                                        { join => MT::ObjectAsset->join_on(undef, {asset_id => \'= asset_id', object_ds => 'entry', object_id => $id })});
+           foreach my $asset (@assets) {
+               my $asset_1;
+               if ($asset->class eq 'image') {
+                   $asset_1 = {asset_id => $asset->id, asset_name => $asset->file_name, asset_thumb => $asset->thumbnail_url(Width=>100)};
+               } else {
+                   $asset_1 = {asset_id => $asset->id, asset_name => $asset->file_name};
+               }
+               push @{$assets}, $asset_1;
+           }
+       }
+       $param->{asset_loop} = $assets;
+    }
+    
     ## Load categories and process into loop for category pull-down.
     require MT::Placement;
     my $cat_id = $param->{category_id};
@@ -1093,7 +1135,7 @@ sub preview {
           };
     }
     for my $data (
-        qw( authored_on_date authored_on_time basename_manual basename_old category_ids tags )
+        qw( authored_on_date authored_on_time basename_manual basename_old category_ids tags include_asset_ids )
       )
     {
         push @data,
@@ -1149,6 +1191,19 @@ sub cfg_entry {
     my $blog_id = scalar $q->param('blog_id');
     return $app->return_to_dashboard( redirect => 1 )
       unless $blog_id;
+    #
+    # User must have can_edit_config, can_administer_blog, or can_set_publish_paths
+    # in order to see the Entry Settings page.
+    #   
+    my $perms      = $app->permissions;
+    return $app->error( $app->translate('Permission denied.') )
+      unless $app->user->is_superuser()
+      || (
+        $perms
+        && (   $perms->can_edit_config
+            || $perms->can_administer_blog
+            || $perms->can_set_publish_paths )
+      );      
     $q->param( '_type', 'blog' );
     $q->param( 'id',    scalar $q->param('blog_id') );
     $app->forward("view",
@@ -1397,6 +1452,37 @@ $ao
         )
       );
 
+    ## look if any assets have been included/removed from this entry
+    require MT::Asset;
+    require MT::ObjectAsset;
+    my $include_asset_ids = $app->param('include_asset_ids');
+    my @asset_ids = split(',', $include_asset_ids);
+    my $obj_assets = ();
+    my @obj_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $obj->id });
+    foreach my $obj_asset (@obj_assets) {
+        my $asset_id = $obj_asset->asset_id;
+        $obj_assets->{$asset_id} = 1;
+    }
+    my $seen = ();
+    foreach my $asset_id (@asset_ids) {
+        my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset_id, object_ds => 'entry', object_id => $obj->id });
+        unless ($obj_asset) {
+            my $obj_asset = new MT::ObjectAsset;
+            $obj_asset->blog_id($blog_id);
+            $obj_asset->asset_id($asset_id);
+            $obj_asset->object_ds('entry');
+            $obj_asset->object_id($obj->id);
+            $obj_asset->save;
+        }
+        $seen->{$asset_id} = 1;
+    }
+    foreach my $asset_id (keys %{$obj_assets}) {
+        unless ($seen->{$asset_id}) {
+            my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset_id, object_ds => 'entry', object_id => $obj->id });
+            $obj_asset->remove;
+        }
+    }
+
     my $message;
     if ($is_new) {
         $message =
