@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Image.pm 3455 2009-02-23 02:29:31Z auno $
+# $Id: Image.pm 4137 2009-08-08 01:52:07Z dphillips $
 
 package MT::Image;
 
@@ -411,7 +411,7 @@ sub scale {
     my $image = shift;
     my($w, $h) = $image->get_dimensions(@_);
     my $src = $image->{gd};
-    my $gd = GD::Image->new($w, $h);
+    my $gd = GD::Image->new($w, $h, 1);  # True color image (24 bit)
     $gd->copyResampled($src, 0, 0, 0, 0, $w, $h, $image->{width}, $image->{height});
     ($image->{gd}, $image->{width}, $image->{height}) = ($gd, $w, $h);
     wantarray ? ($image->blob, $w, $h) : $image->blob;
@@ -422,7 +422,7 @@ sub crop {
     my %param = @_;
     my ($size, $x, $y) = @param{qw( Size X Y )};
     my $src = $image->{gd};
-    my $gd = GD::Image->new($size, $size);
+    my $gd = GD::Image->new($size, $size, 1);  # True color image (24 bit)
     $gd->copy($src, 0, 0, $x, $y, $size, $size);
     ($image->{gd}, $image->{width}, $image->{height}) = ($gd, $size, $size);
     wantarray ? ($image->blob, $size, $size) : $image->blob;
