@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Object.pm 3774 2009-06-01 21:52:50Z jmarcotte $
+# $Id: Object.pm 5087 2009-11-20 19:56:19Z kshay $
 
 package MT::Object;
 
@@ -25,8 +25,8 @@ sub install_pre_init_properties {
     @PRE_INIT_PROPS = ();
 
     foreach my $def (@PRE_INIT_META) {
-        my ($class, $meta) = @$def;
-        $class->install_meta($meta);
+        my ($class, $meta, $which) = @$def;
+        $class->install_meta($meta, $which);
     }
     @PRE_INIT_META = ();
 }
@@ -53,9 +53,12 @@ sub install_properties {
     }
 
     my %meta;
+    my %summary;
 
     my $super_props = $class->SUPER::properties();
-    $props->{meta} = 1 if $super_props && $super_props->{meta};
+    for my $which (qw( meta summary )) {
+        $props->{$which} = 1 if $super_props && $super_props->{$which};
+    }
 
     if ($props->{meta}) {
         # yank out any meta columns before we start working on column_defs
@@ -135,7 +138,15 @@ sub install_properties {
     } else {
         $type_id = $props->{datasource};
     }
-
+    
+    if ($props->{summary}) {
+        my $type_summaries = MT->registry('summaries', $type_id);
+        %summary = map { $_ => 
+                ($type_summaries->{$_}->{type} =~ /(string|integer)/)
+                ? "$1 indexed meta" : "$type_summaries->{$_}->{type} meta"
+            } keys %$type_summaries;
+    }
+    
     $props->{get_driver} ||= sub {
         require MT::ObjectDriverFactory;
         my $coderef = MT::ObjectDriverFactory->driver_for_class($class);
@@ -234,9 +245,18 @@ sub install_properties {
 
     # inherit parent's metadata setup
     if ($props->{meta}) { # if ($super_props && $super_props->{meta_installed}) {
-        $class->install_meta({ ( %meta ? ( column_defs => \%meta ) : ( columns => [] ) ) });
+        $class->install_meta(
+            { ( %meta ? ( column_defs => \%meta ) : ( columns => [] ) ) },
+            'meta'
+        );
         $class->add_trigger( post_remove => \&remove_meta );
     }
+    if ($props->{summary}) { # if ($super_props && $super_props->{meta_installed}) {
+        $class->install_meta(
+            { ( %summary ? ( column_defs => \%summary ) : ( columns => [] ) ) },
+            'summary'
+        );
+    }
 
     return $props;
 }
@@ -392,29 +412,37 @@ sub add_class {
 sub new {
     my $class = shift;
     my $obj = $class->SUPER::new(@_);
-    if ($obj->properties->{meta_installed}) {
-        $obj->init_meta();
+    for my $which (qw( meta summary )) {
+        if ($obj->properties->{$which . '_installed'}) {
+            $obj->init_meta($which);
+        }
     }
     return $obj;
 }
 
 sub init_meta {
     my $obj = shift;
-    require MT::Meta::Proxy;
-    $obj->{__meta} = MT::Meta::Proxy->new($obj);
+    my ($which) = @_;
+    $which ||= 'meta';
+    my $class = 'MT::' . ucfirst($which) . '::Proxy';
+    eval("require $class;");
+    $obj->{"__$which"} = $class->new($obj);
 }
 
 sub install_meta {
     my $class = shift;
-    my ($params) = @_;
+    my ($params, $which) = @_;
+    $which ||= 'meta';
+    my $installed = $which . '_installed';
+    my $meta_class = 'MT::' . ucfirst($which);
     if ( ( $class ne 'MT::Config' ) && (!$MT::plugins_installed) ) {
-        push @PRE_INIT_META, [$class, $params];
+        push @PRE_INIT_META, [$class, $params, $which];
         return;
     }
 
-    require MT::Meta;
+    eval("require $meta_class");
     my $pkg = ref $class || $class;
-    if (!$pkg->SUPER::properties->{meta_installed}) {
+    if (!$pkg->SUPER::properties->{$installed}) {
         $pkg->add_trigger( post_save => \&_post_save_save_metadata );
         $pkg->add_trigger( post_load => \&_post_load_initialize_metadata );
     }
@@ -441,7 +469,7 @@ sub install_meta {
             $type =~ s/\s.*//; # take first keyword, ignoring anything after
             $type .= '_indexed'
                 if $cols->{$col} =~ m/\bindexed\b/;
-            $type = MT::Meta->normalize_type($type);
+            $type = $meta_class->normalize_type($type);
 
             push @{ $params->{fields} }, {
                 name => $col,
@@ -451,19 +479,20 @@ sub install_meta {
         }
     }
 
-    $params->{datasource} ||= $class->datasource . '_meta';
+    $params->{datasource} ||= $class->datasource . "_$which";
 
-    if ($props->{meta_installed} && !@{ $params->{fields} }) {
+    if ($props->{$installed} && !@{ $params->{fields} }) {
         return 1;
     }
 
-    if (my $fields = MT::Meta->install($pkg, $params)) {
+    if (my $fields = $meta_class->install($pkg, $params, $which)) {
         # we may have inherited meta fields so lets update with
         # the fields returned by MT::Meta
-        $props->{fields}->{$_} = $fields->{$_} for keys %$fields;
+        my $which_fields = ($which eq 'meta') ? 'fields' : 'summaries';
+        $props->{$which_fields}->{$_} = $fields->{$_} for keys %$fields;
     }
 
-    return $props->{meta_installed} = 1;
+    return $props->{$installed} = 1;
 }
 
 sub meta_args {
@@ -509,43 +538,93 @@ sub meta_args {
     };
 }
 
+sub summary_args {
+    my $class = shift;
+    my $id_field = $class->datasource . '_id';
+    return {
+        key         => $class->datasource,
+        column_defs => {
+            $id_field         => 'integer not null',
+            type              => 'string(255) not null',
+            class             => 'string(75) not null',
+            vchar_idx         => 'string(255)',
+            vinteger_idx      => 'integer',
+            vblob             => 'blob',
+            vclob             => 'text',
+            expired           => 'smallint',
+        },
+        columns => [ $id_field, qw(
+            type
+            class
+            vchar_idx
+            vinteger_idx
+            vblob
+            vclob
+            expired
+        ) ],
+        indexes => {
+            id_class    => { columns => [ $id_field, 'class' ] },
+            class_vchar => { columns => [ 'class', 'vchar_idx'     ] },
+            class_vint  => { columns => [ 'class', 'vinteger_idx'  ] },
+        },
+        primary_key => [ $id_field, 'type' ],
+    };
+}
+
 sub has_meta {
     my $obj = shift;
     return $obj->is_meta_column(@_) if @_;
     return $obj->properties->{meta_installed} ? 1 : 0;
 }
 
+sub has_summary {
+    my $obj = shift;
+    return $obj->is_summary(@_) if @_;
+    return $obj->properties->{summary_installed} ? 1 : 0;
+}
+
 sub _post_load_initialize_metadata {
     my $obj = shift;
-    if (defined $obj && $obj->properties->{meta_installed}) {
-        $obj->init_meta();
-        $obj->{__meta}->set_primary_keys($obj);
+    for my $which (qw( meta summary )) {
+        if (defined $obj && $obj->properties->{$which . '_installed'}) {
+            $obj->init_meta($which);
+            $obj->{"__$which"}->set_primary_keys($obj);
+        }
     }
 }
 
 sub is_meta_column {
     my $obj = shift;
-    my ($field) = @_;
-
+    my ($field, $which) = @_;
+    $which ||= 'meta';
+    my $which_fields = ($which eq 'meta') ? 'fields' : 'summaries';
     my $props = $obj->properties;
-    return unless $props->{meta_installed};
+    return unless $props->{$which . '_installed'};
 
-    my $meta = $obj->meta_pkg;
-    return 1 if $props->{fields}{$field};
+    my $meta = $obj->meta_pkg($which);
+    return 1 if $props->{$which_fields}{$field};
 
     return;
 }
 
+sub is_summary {
+    my $obj = shift;
+    $obj->is_meta_column($_[0], 'summary');
+}
+
 sub meta_pkg {
     my $class = shift;
+    my ($which) = @_;
+    $which ||= 'meta';
+    my $pkg = $which . '_pkg';
     my $props = $class->properties;
-    return unless $props->{meta}; # this only works for meta-enabled classes
+    return unless $props->{$which}; # this only works for meta-enabled classes
 
-    return $props->{meta_pkg} if $props->{meta_pkg};
+    return $props->{$pkg} if $props->{$pkg};
 
     my $meta = ref $class || $class;
-    $meta .= '::Meta';
-    return $props->{meta_pkg} = $meta;
+    $meta .= '::' . ucfirst($which);
+    return $props->{$pkg} = $meta;
 }
 
 sub has_column {
@@ -563,6 +642,10 @@ sub _post_save_save_metadata {
         $obj->{__meta}->save;
         $orig_obj->{__meta} = $obj->{__meta};
     }
+    if (defined $obj && exists $obj->{__summary}) {
+        $obj->{__summary}->set_primary_keys($obj);
+        $orig_obj->{__summary} = $obj->{__summary};
+    }
 }
 
 sub meta {
@@ -577,6 +660,15 @@ sub meta {
          :                   $obj->{__meta}->get_hash;
 }
 
+sub summary {
+    my $obj = shift;
+    my ( $terms, $value ) = @_;
+    $obj->{__summary}->set_primary_keys($obj);
+    return undef if (!$obj->{__summary});
+    return $obj->{__summary}->set( $terms, $value ) if (scalar @_ == 2);
+    return $obj->{__summary}->get($terms);
+}
+
 sub meta_obj {
     my $obj = shift;
     return $obj->{__meta};
@@ -596,6 +688,12 @@ sub column_func {
             $obj->{__meta}->set($col, @_);
         }
         else {
+            # Repopulate meta fields if they are missing; this fixes an issue with memcached caching an incomplete object
+            if (! exists $obj->{__meta}->{__objects}->{$col}) {
+              my $proxy = $obj->{__meta};
+              $proxy->set_primary_keys($obj) if $obj->has_primary_key;
+              $proxy->load_objects($col);
+            }
             $obj->{__meta}->get($col);
         }
     };
@@ -719,6 +817,7 @@ sub save {
     if (my $err = $@) {
         return $obj->error($err);
     }
+    delete $obj->{__orig_value};
     return $res;
 }
 
@@ -726,8 +825,13 @@ sub remove {
     my $obj = shift;
     my(@args) = @_;
     if (!ref $obj) {
-        $obj->remove_meta( @args ) if $obj->has_meta;
+        for my $which (qw( meta summary )) {
+            my $meth = "remove_$which";
+            my $has = "has_$which";
+            $obj->$meth( @args ) if $obj->$has;
+        }
         $obj->remove_scores( @args ) if $obj->isa('MT::Scorable');
+        MT->run_callbacks($obj . '::pre_remove_multi', @args);
         return $obj->driver->direct_remove($obj, @args);
     } else {
         return $obj->driver->remove($obj, @args);
@@ -889,13 +993,16 @@ sub table_name {
 
 sub clone_all {
     my $obj = shift;
+    my ($param) = @_;
     my $clone = $obj->SUPER::clone_all();
     if ($clone->properties->{meta_installed}) {
         $clone->init_meta();
         $clone->meta( $obj->meta );
-        for my $meta ( keys %{ $clone->{__meta}->{__objects} } ) {
-            $clone->{__meta}->{__objects}{$meta}->{changed_cols}
-                = $obj->{__meta}->{__objects}->{$meta}->{changed_cols} || {};
+        if (!$param || !ref($param) || (ref($param) ne 'HASH') || !$param->{wantmeta}) {
+            for my $meta ( keys %{ $clone->{__meta}->{__objects} } ) {
+                $clone->{__meta}->{__objects}{$meta}->{changed_cols}
+                    = $obj->{__meta}->{__objects}->{$meta}->{changed_cols} || {};
+            }
         }
     }
     return $clone;
@@ -904,7 +1011,9 @@ sub clone_all {
 sub clone {
     my $obj = shift;
     my($param) = @_;
-    my $clone = $obj->clone_all();
+    # pass wantmeta to indicate that it should clone all metadata regardless of
+    # changed status
+    my $clone = $obj->clone_all({ wantmeta => 1 });
 
     ## If the caller has listed a set of columns not to copy to the clone,
     ## delete them from the clone.
@@ -970,7 +1079,12 @@ sub join_on {
 
 sub remove_meta {
     my $obj = shift;
-    my $mpkg = $obj->meta_pkg or return;
+    my $which;
+    if ($_[0] && !ref($_[0]) && (($_[0] || '') =~ /meta|summary/)) {
+        $which = shift;
+    }
+    $which ||= 'meta';
+    my $mpkg = $obj->meta_pkg($which || 'meta') or return;
     if ( ref $obj ) {
         my $id_field = $obj->datasource . '_id';
         return $mpkg->remove({ $id_field => $obj->id });
@@ -1003,6 +1117,11 @@ sub remove_meta {
     }
 }
 
+sub remove_summary {
+    my $obj = shift;
+    $obj->remove_meta('summary', @_);
+}
+
 sub remove_scores {
     my $class = shift;
     require MT::ObjectScore;
@@ -1077,8 +1196,11 @@ sub set_by_key {
 sub deflate {
     my $obj = shift;
     my $data = $obj->SUPER::deflate();
-    if ($obj->has_meta()) {
-        $data->{meta} = $obj->{__meta}->deflate_meta();
+    for my $which (qw( meta summary )) {
+        my $meth = "has_$which";
+        if ($obj->$meth()) {
+            $data->{$which} = $obj->{"__$which"}->deflate_meta();
+        }
     }
     return $data;
 }
@@ -1087,8 +1209,13 @@ sub inflate {
     my $class = shift;
     my ($data) = @_;
     my $obj = $class->SUPER::inflate(@_);
-    if ($class->has_meta()) {
-        $obj->{__meta}->inflate_meta($data->{meta});
+    for my $which (qw( meta )) {
+        # only inflate meta; don't inflate summary as this create zombie objects in the cache
+        # that prevent real MT::*::Summary objects from being used
+        my $meth = "has_$which";
+        if ($class->$meth()) {
+            $obj->{"__$which"}->inflate_meta($data->{$which});
+        }
     }
     return $obj;
 }
@@ -2495,7 +2622,9 @@ the class.
 For object classes that have metadata storage, this method will initialize
 the metadata member.
 
-=item * Class->install_meta( \%meta_properties )
+=item * Class->install_meta( \%meta_properties, [ $which ] )
+
+Called to register metadata or summary properties on a particular class. If 'summary' is passed in the optional C<$which> parameter, summary properties will be installed, and metadata properties otherwise.
 
 Called to register metadata properties on a particular class. The
 C<%meta_properties> may contain an arrayref of 'columns', or a hashref
@@ -2531,25 +2660,43 @@ and C<%args> parameters.
 
 Returns the L<MT::Object> class
 
-=item * Class->meta_pkg()
+=item * Class->meta_pkg( [ $which ] )
 
-Returns the Perl package name for storing it's metadata objects.
+Returns the Perl package name for storing the class's metadata objects,
+or the class's summary objects if 'summary' is passed in C<$which>.
 
 =item * Class->meta_args
 
 Returns the source of a Perl package declaration that is loaded to
 declare and process metadata objects for the C<Class>.
 
+=item * Class->summary_args
+
+Returns the source of a Perl package declaration that is loaded to 
+declare and process summary data objects for the C<Class>.
+
 =item * Class->has_meta( [ $name ] )
 
 Returns a boolean as to whether the class has metadata when called
 without a parameter, or whether there exists a metadata column
 of the given C<$name>.
 
-=item * Class->is_meta_column( $name )
+=item * Class->has_summary( [ $name ] )
+
+Returns a boolean as to whether the class supports summaries when 
+called without a parameter, or whether a summary type of the given 
+C<$name> is defined for the class.
+
+=item * Class->is_meta_column( $name, [ $which ] )
 
 Returns a boolean as to whether the class has a meta column named
-C<$name>.
+C<$name>, or a summary of type C<$name> if 'summary' is passed
+in C<$which>.
+
+=item * Class->is_summary( $name )
+
+Returns a boolean as to whether a summary type of the given C<$name> 
+is defined for the class.
 
 =back
 
