@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: DBI.pm 3783 2009-06-03 13:46:10Z fumiakiy $
+# $Id: DBI.pm 4020 2009-07-15 16:14:27Z plim $
 
 package MT::ObjectDriver::Driver::DBI;
 
@@ -19,7 +19,12 @@ sub init {
     $param{prefix} ||= 'mt_';
     $driver->SUPER::init(%param);
     my $opts = $driver->connect_options || {};
-    $opts->{RaiseError} = 0;
+    
+    require MT;
+    my $mt = MT->instance;
+    my $cfg = $mt->config;
+    $opts->{RaiseError} = $cfg->DBIRaiseError;
+    
     $driver->connect_options($opts);
     $driver;
 }
