@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: ObjectDriverFactory.pm 3466 2009-02-27 19:55:09Z bchoate $
+# $Id: ObjectDriverFactory.pm 4148 2009-08-19 07:39:07Z dphillips $
 
 package MT::ObjectDriverFactory;
 
@@ -109,6 +109,21 @@ sub configure {
 }
 
 sub cleanup {
+    if ( my $driver = $MT::Object::DRIVER ) {
+        if ( my $dbh = $driver->dbh ) {
+            $dbh->disconnect;
+            $driver->dbh(undef);
+        }
+        $MT::Object::DRIVER = undef;
+        $MT::Object::DBI_DRIVER = undef;
+    }
+    foreach my $driver (@drivers) {
+        if ( my $dbh = $driver->dbh ) {
+            $dbh->disconnect;
+            $driver->dbh(undef);
+        }
+    }
+    @drivers = ();
     undef $DRIVER;
 }
 
