@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: ObjectScore.pm 3455 2009-02-23 02:29:31Z auno $
+# $Id: ObjectScore.pm 4045 2009-07-21 19:53:59Z kshay $
 
 package MT::ObjectScore;
 
@@ -63,6 +63,15 @@ sub scored_by {
     });
 }
 
+sub score {
+    my $objectscore = shift;
+    if ( scalar @_ ) {
+        $objectscore->{__orig_value}->{score} = $objectscore->score
+            unless exists( $objectscore->{__orig_value}->{score} );
+    }
+    return $objectscore->SUPER::score( @_ );
+}
+
 1;
 __END__
 
