@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Scorable.pm 3455 2009-02-23 02:29:31Z auno $
+# $Id: Scorable.pm 4077 2009-07-23 15:38:57Z kshay $
 
 # An interface for any MT::Object that wishes to utilize rating/scoring themselves
 
@@ -46,7 +46,7 @@ sub get_score {
 
 sub set_score {
     my $obj = shift;
-    my ( $namespace, $user, $score, $overwrite ) = @_;
+    my ( $namespace, $user, $score, $overwrite, $blog ) = @_;
 
     return $obj->error( MT->translate('Object must be saved first.') )
       unless $obj->id;
@@ -73,7 +73,16 @@ sub set_score {
         $s = MT::ObjectScore->new;
         $s->set_values($term);
     }
+    $s->{__orig_value}->{score} = $s->score
+        unless exists( $s->{__orig_value}->{score} );
     $s->score($score);
+    if ($blog) {
+		$blog = MT->model('blog')->load($blog) unless ref($blog);
+    	require MT::Util;
+        my $ts = MT::Util::epoch2ts($blog, time);
+        $s->created_on($ts);
+        $s->modified_on($ts);
+    }
     $s->save
       or return $obj->error(
         MT->translate(
