@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: TheSchwartz.pm 3733 2009-05-18 20:27:46Z jmarcotte $
+# $Id: TheSchwartz.pm 4043 2009-07-21 19:33:50Z plim $
 
 package MT::TheSchwartz;
 
@@ -294,7 +294,7 @@ sub work_periodically {
 
 our %persistent;
 BEGIN {
-    %persistent = map { $_ => 1 } qw( MT::Callback MT::Task MT::Plugin MT::Component MT::ArchiveType MT::TaskMgr MT::WeblogPublisher MT::Serializer TheSchwartz::Job TheSchwartz::JobHandle );
+    %persistent = map { $_ => 1 } qw( MT::Callback MT::Task MT::Plugin MT::Component MT::ArchiveType MT::TaskMgr MT::WeblogPublisher MT::Serialize TheSchwartz::Job TheSchwartz::JobHandle );
 }
 sub leak_report {
     my ($start, $pre, $post) = @_;
