@@ -1,5 +1,5 @@
 /*
-# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2010 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -123,6 +123,9 @@ MT.App = new Class( MT.App, {
                 this.calendar.open(
                     {
                         date: DOM.getElement( "created-on" ).value
+                        + 'T'
+                        + DOM.getElement( "entry_form" ).elements['authored_on_time'].value
+                        + 'Z'
                     },
                     this.getIndirectMethod( "handleCreatedOnDate" ),
                     event.commandElement
